/** \file clamp_util.h
 * \brief Defines some utility functions and macros, used by BBP and CBP
 */
#ifndef ____defined_libdai_clamp_util_h__
#define ____defined_libdai_clamp_util_h__

#include <vector>
#include <dai/daialg.h>

/// Like foreach, but for traversing a half-open interval \f$[0,n)\f$
#define foreach_size_t(i, n) for(size_t i=0; i<n; i++)
/// Produces a random number in the half-open interval \f$[0,n)\f$
#define rnd_multi(n) rnd_int(0,(n)-1)

namespace dai {

/// clamp a variable to have one of a list of values
void clampVar( FactorGraph &g, size_t i, const std::vector<size_t> &xis );

/// clamp a factor to have one of a list of values
void clampFactor( FactorGraph &g, size_t I, const std::vector<size_t> &xIs );

/// \brief Given a sorted vector of states \a xis and total state count \a n_states,
/// return a vector of states not in \a xis
std::vector<size_t> complement(std::vector<size_t>& xis, size_t n_states);

/// Maximum of two Reals
Real max(Real x,Real y);

/// Computes \code exp(lz)/(exp(lz)+exp(cmp_lz))\endcode
Real unSoftMax(Real lz, Real cmp_lz);

/// Log of sum of exponents
Real logSumExp(Real lz, Real cmp_lz);

/// Compute total distance of first \a nv factors in each vector
Real dist(const std::vector<Factor>& b1, const std::vector<Factor>& b2, size_t nv);

}

#endif /* ____defined_libdai_clamp_util_h__ */
