#include <dai/clamp_util.h>

using namespace std;

namespace dai {

/// clamp a variable to have one of a set of values
void clampVar(FactorGraph &g, size_t i, const vector<size_t> &is ) {
    Var n=g.var(i);
    Factor mask_n(n,0.0);

    foreach(size_t i, is) { assert( i <= n.states() ); mask_n[i] = 1.0; }

    for( size_t I = 0; I < g.nrFactors(); I++ ) 
        if( g.factor(I).vars().contains( n ) )
          g.factor(I) *= mask_n;
}

/// clamp a factor to have one of a set of values
void clampFactor(FactorGraph &g, size_t I, const vector<size_t> &is) {
    size_t st = g.factor(I).states();
    Prob mask_n(st,0.0);

    foreach(size_t i, is) { assert( i <= st ); mask_n[i] = 1.0; }

    g.factor(I).p() *= mask_n;
}


vector<size_t> complement(vector<size_t>& xis, size_t n_states) {
    vector<size_t> cmp_xis(0);
    size_t j=0;
    for(size_t xi=0; xi<n_states; xi++) {
        while(j<xis.size() && xis[j]<xi) j++;
        if(j>=xis.size() || xis[j]>xi) cmp_xis.push_back(xi);
    }
    assert( xis.size()+cmp_xis.size() == n_states );
    return cmp_xis;
}

Real max(Real x,Real y) { return x>y?x:y; }

Real unSoftMax(Real lz, Real cmp_lz) {
    double m = max(lz, cmp_lz);
    lz -= m; cmp_lz -= m;
    double p = exp(lz)/(exp(lz)+exp(cmp_lz));
    return p;
}

Real logSumExp(Real lz, Real cmp_lz) {
    double m = max(lz, cmp_lz);
    lz -= m; cmp_lz -= m;
    return m+log(exp(lz)+exp(cmp_lz));
}

Real dist(const vector<Factor>& b1, const vector<Factor>& b2, size_t nv) {
    Real d=0.0;
    for(size_t k=0; k<nv; k++) {
        d += dist( b1[k], b2[k], Prob::DISTLINF );
    }
    return d;
}

}
